package syntax.expressions;
import parsing.*;


public abstract class Expression extends RightHandSide
{
	String type;
	
	Value result;
	
	public static Expression read(Stream stream, int minPriority)
	{

		Expression left;
		Symbol     op;
		Expression right;
		
		left = readSingleExpr(stream);
		
		while(true)  // while the following operator has higher priority, bundle them together
		{

			try {
				op = Symbol.read(stream.after(left));

			}
			catch (ParsingException pe) {

				return left;
			}
			
			if (OpDef.operators.containsKey(op.name) == false)
			{

				throw new ParsingException(stream, "Unknown operator: '" + op.name + "'.");
			}
			
			OpDef opDef = OpDef.operators.get(op.name);
 			int p = opDef.priority;
			
			if (p > minPriority)
			{

				right = Expression.read(stream.after(op), p);
				
				// ok, we can already bundle them... 

				left = new OpCall(op, left, right);
				// ...and continue parsing the rest of the expression

			}
			else
			{

				return left;
			}
			
		}
	}


	

	private static Expression readSingleExpr(Stream stream)
	{

		try {
			return Value.read(stream);
		}
		catch (ParsingException pe1) {
			try {
				return FunctionCall.read(stream);
			}
			catch (ParsingException pe2) {	
				ParsingException def = new ParsingException(stream, "Expression could not be matched.");
				throw ParsingException.latest(new ParsingException[] {def, pe1, pe2});
			}
		}
	}

	
	public abstract Value eval();
}
