package syntax.expressions;
import parsing.*;


public class StructuredValue extends Value
{
	Value[] slots;


	public StructuredValue(String type, Value[] slots, SrcLoc start, SrcLoc end)
	{

		this.type = type;
		this.slots = slots;
		this.start = start;
		this.end = end;
	}
	
	public boolean equals(StructuredValue other)
	{
		return this.type.equals(other.type) && this.slots.equals(other.slots);
	}

	public static StructuredValue read(Stream stream)
	{
		String tok = StreamHandler.grabToken(stream);

		// try to parse an int
		if (tok.matches("[A-Z].*"))
		{
			SrcLoc s = stream.loc;
			SrcLoc e = stream.loc.after(tok.length() - 1);
			return new StructuredValue(tok, new Value[]{}, s, e);
		}
			
		throw new ParsingException(stream, "Structured value could not be matched.");
		
	}

	public String toString()
	{
		String ans = type + "\n";
		for (Value v : slots)
			ans += indent(v.toString()) + "\n";
		
		return ans;
	}

	String indent(String s)
	{
		// This is a temporary version
		// TODO: split string, indent all lines and merge
		return ("\t" + s);
	}
}
