package syntax.expressions;

import java.util.Vector;
import parsing.*;


public class Tuple<T> extends Value {

	public Vector<T> members = new Vector<T>();
	
	Tuple()
	{
		type = "Tuple";
	}
	
	
	
	public static Tuple<Expression> readExpressionTuple(Stream stream)
	{
		Tuple<Expression> tuple = new Tuple<Expression>();
		Token lp = Token.read("(", stream);
		
		Expression e = null;
		
		try
		{
			e = Expression.read(stream.after(lp), 0);
			tuple.members.add(e);
		}
		catch(ParsingException pe)
		{
			Token rp = Token.read(")", stream.after(e));
			
			tuple.start = lp.start;
			tuple.end = rp .end;
			
			return tuple;
		}
			
		while(true)
		{
			Token comma;
			try
			{
				comma = Token.read(",", stream.after(e));
			}
			catch(Exception pe)
			{
				break;
			}
			
			e = Expression.read(stream.after(comma), 0);
			tuple.members.add(e);
		}
			
		Token rp = Token.read(")", stream.after(e));
		
		if (tuple.members.size() == 1)
		{
			tuple.type = tuple.members.get(0).type;
		}
		
		tuple.start = lp.start;
		tuple.end = rp .end;
		
		return tuple;
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	public static Tuple<Symbol> readSymbolTuple(Stream stream)
	{
		Tuple<Symbol> tuple = new Tuple<Symbol>();
		Token lp = Token.read("(", stream);
		
		Symbol s = null;
		
		try
		{
			s = Symbol.read(stream.after(lp));
			tuple.members.add(s);
		}
		catch(Exception e)
		{
			Token rp = Token.read(")", stream.after(s));
			
			tuple.start = lp.start;
			tuple.end = rp .end;
			
			return tuple;
		}
			
		while(true)
		{
			Token comma;
			try
			{
				comma = Token.read(",", stream.after(s));
			}
			catch(Exception e)
			{
				break;
			}
			
			s = Symbol.read(stream.after(comma));
			tuple.members.add(s);
		}
			
		Token rp = Token.read(")", stream.after(s));
		
		tuple.start = lp.start;
		tuple.end = rp .end;
		
		return tuple;
	}
	
	
	public static <T> String asString(Vector<T> elems)
	{
		if (elems.isEmpty())
		{
			return "()";
		}
		
		String res = "(";
		res += elems.get(0).toString();
		
		for (int i = 1; i < elems.size(); i++)
		{
			res += ", " + elems.get(i).toString();
		}
		
		res += ")";
		return res;
	}
	
	public Value eval()
	{
		return this;
	}

	public String toString()
	{
		return asString(members);
	}

}

